# ---------------------------------------------------------------------
# Copyright (c) 2024 Qualcomm Innovation Center, Inc. All rights reserved.
# SPDX-License-Identifier: BSD-3-Clause
# ---------------------------------------------------------------------
import numpy as np
import pytest

from qai_hub_models.models._shared.body_detection.app import BodyDetectionApp
from qai_hub_models.models.face_body_net.demo import main as demo_main
from qai_hub_models.models.face_body_net.model import (
    MODEL_ASSET_VERSION,
    MODEL_ID,
    FaceBodyNet,
)
from qai_hub_models.utils.asset_loaders import CachedWebModelAsset, load_raw_file
from qai_hub_models.utils.testing import skip_clone_repo_check

INPUT_IMAGE_ADDRESS = CachedWebModelAsset.from_asset_store(
    MODEL_ID, MODEL_ASSET_VERSION, "test_image.jpg"
)
GROUND_TRUTH_RESULT = CachedWebModelAsset.from_asset_store(
    MODEL_ID, MODEL_ASSET_VERSION, "ground_truth.txt"
)


@skip_clone_repo_check
def test_task() -> None:
    app = BodyDetectionApp(FaceBodyNet.from_pretrained())
    result = app.detect(INPUT_IMAGE_ADDRESS, 640, 640, 0.5)
    assert len(result) == 4


@pytest.mark.trace
@skip_clone_repo_check
def test_trace() -> None:
    app = BodyDetectionApp(FaceBodyNet.from_pretrained().convert_to_torchscript())
    result = app.detect(INPUT_IMAGE_ADDRESS, 640, 640, 0.5)
    gt = load_raw_file(GROUND_TRUTH_RESULT)
    expected = np.array(gt.split(), dtype=int)
    result = result.astype(int)
    for t in range(5):
        assert result[0][t] == expected[t]


@skip_clone_repo_check
def test_demo() -> None:
    demo_main(is_test=True)
