# ---------------------------------------------------------------------
# Copyright (c) 2024 Qualcomm Innovation Center, Inc. All rights reserved.
# SPDX-License-Identifier: BSD-3-Clause
# ---------------------------------------------------------------------
# THIS FILE WAS AUTO-GENERATED. DO NOT EDIT MANUALLY.

from __future__ import annotations

from contextlib import nullcontext
from typing import ContextManager, cast
from unittest import mock

import pytest
import qai_hub as hub
import torch

from qai_hub_models.models.common import ExportResult, TargetRuntime
from qai_hub_models.models.mobile_vit import Model
from qai_hub_models.models.mobile_vit import export as model_export_module
from qai_hub_models.models.mobile_vit.export import export_model
from qai_hub_models.scorecard import (
    ScorecardCompilePath,
    ScorecardDevice,
    ScorecardProfilePath,
)
from qai_hub_models.scorecard.device import cs_8_gen_3, cs_universal
from qai_hub_models.scorecard.execution_helpers import (
    get_async_job_cache_name,
    get_async_job_id,
    get_compile_parameterized_pytest_config,
    get_profile_parameterized_pytest_config,
    pytest_device_idfn,
)
from qai_hub_models.utils.asset_loaders import load_yaml
from qai_hub_models.utils.testing import (
    get_compile_job_ids_file,
    get_inference_job_ids_file,
    get_profile_job_ids_file,
    is_hub_testing_async,
    mock_tabulate_fn,
    verify_io_names,
    write_accuracy,
)


@pytest.mark.compile
def test_verify_io_names() -> None:
    verify_io_names(Model)


@pytest.mark.parametrize(
    "scorecard_path,device",
    get_compile_parameterized_pytest_config(),
    ids=pytest_device_idfn,
)
@pytest.mark.compile
def test_compile(scorecard_path: ScorecardCompilePath, device: ScorecardDevice) -> None:
    runtime = scorecard_path.runtime

    export_result = export_model(
        device=device.execution_device_name,
        chipset=device.chipset,
        skip_downloading=True,
        skip_profiling=True,
        skip_inferencing=True,
        skip_summary=True,
        compile_options=scorecard_path.get_compile_options(),
        target_runtime=runtime,
    )
    assert isinstance(export_result, ExportResult)
    compile_job = export_result.compile_job
    assert compile_job is not None
    if is_hub_testing_async():
        with open(get_compile_job_ids_file(), "a") as f:
            scorecard_path_name = get_async_job_cache_name(
                path=scorecard_path,
                model_id="mobile_vit",
                device=device,
            )
            f.write(f"{scorecard_path_name}: {compile_job.job_id}\n")
    else:
        job_result = compile_job.wait()
        assert job_result.success


@pytest.mark.parametrize(
    "scorecard_path,device",
    get_profile_parameterized_pytest_config(),
    ids=pytest_device_idfn,
)
@pytest.mark.profile
def test_profile(scorecard_path: ScorecardProfilePath, device: ScorecardDevice) -> None:
    runtime = scorecard_path.runtime

    patch: ContextManager = nullcontext()
    if is_hub_testing_async():
        job_ids = load_yaml(get_compile_job_ids_file())
        if job_id := get_async_job_id(
            cache=job_ids,
            path=scorecard_path.compile_path,
            model_id="mobile_vit",
            device=device,
            fallback_to_universal_device=True,
        ):
            job = cast(hub.CompileJob, hub.get_job(job_id))
            if job.get_status().failure:
                pytest.skip("Compile job failed for mobile_vit")
            if not job.get_status().finished:
                pytest.skip("Compile job still running for mobile_vit")
            patch = mock.patch("qai_hub.submit_compile_job", return_value=job)
        else:
            pytest.skip("Compile job skipped for mobile_vit")

    with patch:
        export_result = export_model(
            device=device.execution_device_name,
            chipset=device.chipset,
            skip_downloading=True,
            skip_profiling=False,
            skip_inferencing=True,
            skip_summary=True,
            compile_options=scorecard_path.compile_path.get_compile_options(),
            profile_options=scorecard_path.profile_options,
            target_runtime=runtime,
        )
    assert isinstance(export_result, ExportResult)
    profile_job = export_result.profile_job
    assert profile_job is not None
    if is_hub_testing_async():
        with open(get_profile_job_ids_file(), "a") as f:
            job_name = get_async_job_cache_name(
                path=scorecard_path,
                model_id="mobile_vit",
                device=device,
            )
            f.write(f"{job_name}: {profile_job.job_id}\n")
    else:
        job_result = profile_job.wait()
        assert job_result.success


@pytest.mark.parametrize(
    "runtime",
    [
        TargetRuntime.TFLITE,
        TargetRuntime.QNN,
        TargetRuntime.ONNX,
    ],
)
@pytest.mark.inference
def test_inference(runtime: TargetRuntime) -> None:
    patch: ContextManager = nullcontext()
    if is_hub_testing_async():
        job_ids = load_yaml(get_compile_job_ids_file())
        if job_id := get_async_job_id(
            cache=job_ids,
            path=runtime,
            model_id="mobile_vit",
            device=cs_8_gen_3,
            fallback_to_universal_device=True,
        ):
            job = cast(hub.CompileJob, hub.get_job(job_id))
            if job.get_status().failure:
                pytest.skip("Compile job failed for mobile_vit")
            if not job.get_status().finished:
                pytest.skip("Compile job still running for mobile_vit")
            patch = mock.patch("qai_hub.submit_compile_job", return_value=job)
        else:
            pytest.skip("Compile job skipped for mobile_vit")

    with patch:
        export_result = export_model(
            skip_downloading=True,
            skip_profiling=True,
            skip_inferencing=False,
            skip_summary=True,
            compile_options=ScorecardCompilePath[runtime.name].get_compile_options(),
            profile_options=ScorecardProfilePath[runtime.name].profile_options,
            target_runtime=runtime,
        )
    assert isinstance(export_result, ExportResult)
    inference_job = export_result.inference_job
    assert inference_job is not None
    if is_hub_testing_async():
        with open(get_inference_job_ids_file(), "a") as f:
            job_name = get_async_job_cache_name(
                path=runtime,
                model_id="mobile_vit",
                device=cs_universal,
            )
            f.write(f"{job_name}: {inference_job.job_id}\n")
    else:
        job_result = inference_job.wait()
        assert job_result.success


@pytest.mark.parametrize(
    "runtime",
    [
        TargetRuntime.TFLITE,
        TargetRuntime.QNN,
        TargetRuntime.ONNX,
    ],
)
@pytest.mark.export
def test_val_accuracy(runtime: TargetRuntime) -> None:
    compile_job_ids = load_yaml(get_compile_job_ids_file())
    inference_job_ids = load_yaml(get_inference_job_ids_file())
    compile_job_id = get_async_job_id(
        cache=compile_job_ids,
        path=runtime,
        model_id="mobile_vit",
        device=cs_8_gen_3,
        fallback_to_universal_device=True,
    )
    inference_job_name = get_async_job_cache_name(
        path=runtime,
        model_id="mobile_vit",
        device=cs_universal,
    )
    inference_job_id = inference_job_ids[inference_job_name]
    assert compile_job_id is not None

    compile_job = cast(hub.CompileJob, hub.get_job(compile_job_id))
    inference_job = cast(hub.InferenceJob, hub.get_job(inference_job_id))
    compile_patch = mock.patch("qai_hub.submit_compile_job", return_value=compile_job)
    inference_patch = mock.patch(
        "qai_hub.submit_inference_job", return_value=inference_job
    )
    psnr_values = []

    def _mock_tabulate_fn(df, **kwargs) -> str:
        nonlocal psnr_values
        new_psnr_values, tabulate_results = mock_tabulate_fn(df)
        psnr_values.extend(new_psnr_values)
        return tabulate_results

    tabulate_patch = mock.patch(
        "qai_hub_models.utils.printing.tabulate",
        side_effect=_mock_tabulate_fn,
    )
    with inference_patch, compile_patch, tabulate_patch:
        export_model(target_runtime=runtime, skip_downloading=True, skip_profiling=True)

    write_accuracy("mobile_vit", runtime, psnr_values)


@pytest.mark.parametrize(
    "runtime",
    [
        TargetRuntime.TFLITE,
        TargetRuntime.QNN,
        TargetRuntime.ONNX,
    ],
)
@pytest.mark.export
def test_export(runtime: TargetRuntime) -> None:
    assert is_hub_testing_async(), "Must test export async."

    compile_job_ids = load_yaml(get_compile_job_ids_file())
    profile_job_ids = load_yaml(get_profile_job_ids_file())
    compile_job_id = get_async_job_id(
        cache=compile_job_ids,
        path=runtime,
        model_id="mobile_vit",
        device=cs_8_gen_3,
        fallback_to_universal_device=True,
    )
    assert compile_job_id is not None
    profile_job_name = get_async_job_cache_name(
        path=runtime,
        model_id="mobile_vit",
        device=cs_8_gen_3,
    )
    profile_job_id = profile_job_ids.get(profile_job_name, None)

    compile_job = cast(hub.CompileJob, hub.get_job(compile_job_id))
    profile_job = cast(hub.ProfileJob, hub.get_job(profile_job_id))
    compile_patch = mock.patch("qai_hub.submit_compile_job", return_value=compile_job)
    profile_patch = mock.patch("qai_hub.submit_profile_job", return_value=profile_job)
    with compile_patch, profile_patch:
        export_model(
            target_runtime=runtime, skip_downloading=True, skip_inferencing=True
        )


# Trace the model & upload to hub only once for all module + input pairs.
# This speeds up tests and limits memory leaks for torch jit trace.
@pytest.fixture(scope="module", autouse=True)
def cached_torch_trace_for_export():
    with pytest.MonkeyPatch.context() as mp:
        model_cache = {}
        torch_trace = torch.jit.trace

        def _cached_torch_trace(module, inputs, check_trace=True) -> hub.Model:
            model_key = str(module) + str(
                sorted([f"{x.shape}{x.type()}" for x in inputs])
            )
            model = model_cache.get(model_key, None)
            if not model:
                trace = torch_trace(module, inputs, check_trace=check_trace)
                model = hub.upload_model(trace)
                model_cache[model_key] = model
            assert isinstance(model, hub.Model)
            return model

        mp.setattr(model_export_module.torch.jit, "trace", _cached_torch_trace)
        yield mp
