# ---------------------------------------------------------------------
# Copyright (c) 2024 Qualcomm Innovation Center, Inc. All rights reserved.
# SPDX-License-Identifier: BSD-3-Clause
# ---------------------------------------------------------------------
# THIS FILE WAS AUTO-GENERATED. DO NOT EDIT MANUALLY.

from __future__ import annotations

import time
from collections.abc import Mapping
from contextlib import nullcontext
from typing import ContextManager, cast
from unittest import mock

import numpy as np
import pytest
import qai_hub as hub
import torch

from qai_hub_models.models.common import ExportResult, TargetRuntime
from qai_hub_models.models.resnet18_quantized import Model
from qai_hub_models.models.resnet18_quantized import export as model_export_module
from qai_hub_models.models.resnet18_quantized.export import export_model
from qai_hub_models.scorecard import (
    ScorecardCompilePath,
    ScorecardDevice,
    ScorecardProfilePath,
)
from qai_hub_models.scorecard.device import cs_8_gen_3
from qai_hub_models.scorecard.execution_helpers import (
    get_async_job_cache_name,
    get_async_job_id,
    get_compile_parameterized_pytest_config,
    get_profile_parameterized_pytest_config,
    pytest_device_idfn,
)
from qai_hub_models.utils.asset_loaders import load_yaml
from qai_hub_models.utils.evaluate import evaluate_on_dataset
from qai_hub_models.utils.inference import AsyncOnDeviceModel
from qai_hub_models.utils.testing import (
    append_line_to_file,
    get_and_sync_datasets_cache_dir,
    get_compile_job_ids_file,
    get_dataset_ids_file,
    get_hub_val_dataset,
    get_inference_job_ids_file,
    get_profile_job_ids_file,
    get_quantize_job_ids_file,
    get_torch_val_dataloader,
    is_hub_testing_async,
    mock_first_n,
    mock_get_calibration_data,
    mock_on_device_model_call,
    mock_tabulate_fn,
    verify_io_names,
    write_accuracy,
)


@pytest.mark.compile
def test_verify_io_names() -> None:
    verify_io_names(Model)


@pytest.mark.quantize
def test_quantize() -> None:
    calibration_data_patch = mock.patch(
        "qai_hub_models.models.resnet18_quantized.export.get_calibration_data",
        side_effect=mock_get_calibration_data,
    )
    with calibration_data_patch:
        export_result = export_model(
            device="Samsung Galaxy S24",
            skip_downloading=True,
            skip_compiling=True,
            skip_profiling=True,
            skip_inferencing=True,
            skip_summary=True,
        )
    assert isinstance(export_result, ExportResult)
    quantize_job = export_result.quantize_job
    assert quantize_job is not None
    if is_hub_testing_async():
        with open(get_quantize_job_ids_file(), "a") as f:
            f.write(f"resnet18_quantized: {quantize_job.job_id}\n")
    else:
        job_result = quantize_job.wait()
        assert job_result.success


@pytest.mark.parametrize(
    "scorecard_path,device",
    get_compile_parameterized_pytest_config(model_is_quantized=True),
    ids=pytest_device_idfn,
)
@pytest.mark.compile
def test_compile(scorecard_path: ScorecardCompilePath, device: ScorecardDevice) -> None:
    runtime = scorecard_path.runtime

    compile_patch: ContextManager = nullcontext()
    quantize_patch: ContextManager = nullcontext()
    data_patch: ContextManager = nullcontext()
    if is_hub_testing_async():
        job_ids = load_yaml(get_quantize_job_ids_file())
        if job_id := job_ids.get("resnet18_quantized"):
            job = cast(hub.QuantizeJob, hub.get_job(job_id))
            # Wait 20 minutes from the job start time
            time_left = int(job.date.timestamp() + 1200 - time.time())
            if time_left > 0:
                job.wait(time_left)
            if job.get_status().failure:
                raise ValueError("Quantize job failed for resnet18_quantized")
            if not job.get_status().finished:
                raise ValueError("Quantize job still running for resnet18_quantized")
            quantize_patch = mock.patch("qai_hub.submit_quantize_job", return_value=job)
            # Mock the first call to submit_compile_job and actually call it afterwards
            compile_patch = mock.patch(
                "qai_hub.submit_compile_job",
                side_effect=mock_first_n(hub.submit_compile_job, 1),
            )
            data_patch = mock.patch(
                "qai_hub_models.models.resnet18_quantized.export.get_calibration_data",
                mock.Mock(),
            )
        else:
            raise ValueError("No quantize job found in cache for resnet18_quantized")
    with compile_patch, quantize_patch, data_patch:
        export_result = export_model(
            device=device.execution_device_name,
            chipset=device.chipset,
            skip_downloading=True,
            skip_profiling=True,
            skip_inferencing=True,
            skip_summary=True,
            compile_options=scorecard_path.get_compile_options(model_is_quantized=True),
            target_runtime=runtime,
        )
    assert isinstance(export_result, ExportResult)
    compile_job = export_result.compile_job
    assert compile_job is not None
    if is_hub_testing_async():
        with open(get_compile_job_ids_file(), "a") as f:
            scorecard_path_name = get_async_job_cache_name(
                path=scorecard_path,
                model_id="resnet18_quantized",
                device=device,
            )
            f.write(f"{scorecard_path_name}: {compile_job.job_id}\n")
    else:
        job_result = compile_job.wait()
        assert job_result.success


@pytest.mark.parametrize(
    "scorecard_path,device",
    get_profile_parameterized_pytest_config(model_is_quantized=True),
    ids=pytest_device_idfn,
)
@pytest.mark.profile
def test_profile(scorecard_path: ScorecardProfilePath, device: ScorecardDevice) -> None:
    runtime = scorecard_path.runtime

    patch: ContextManager = nullcontext()
    quantize_patch: ContextManager = nullcontext()
    data_patch: ContextManager = nullcontext()
    if is_hub_testing_async():
        job_ids = load_yaml(get_compile_job_ids_file())
        if job_id := get_async_job_id(
            cache=job_ids,
            path=scorecard_path.compile_path,
            model_id="resnet18_quantized",
            device=device,
            fallback_to_universal_device=True,
        ):
            job = cast(hub.CompileJob, hub.get_job(job_id))
            if job.get_status().failure:
                pytest.skip("Compile job failed for resnet18_quantized")
            if not job.get_status().finished:
                pytest.skip("Compile job still running for resnet18_quantized")
            patch = mock.patch("qai_hub.submit_compile_job", return_value=job)
            quantize_patch = mock.patch(
                "qai_hub.submit_quantize_job", return_value=mock.Mock()
            )
            data_patch = mock.patch(
                "qai_hub_models.models.resnet18_quantized.export.get_calibration_data",
                mock.Mock(),
            )
        else:
            pytest.skip("Compile job skipped for resnet18_quantized")

    with patch, quantize_patch, data_patch:
        export_result = export_model(
            device=device.execution_device_name,
            chipset=device.chipset,
            skip_downloading=True,
            skip_profiling=False,
            skip_inferencing=True,
            skip_summary=True,
            compile_options=scorecard_path.compile_path.get_compile_options(
                model_is_quantized=True
            ),
            profile_options=scorecard_path.profile_options,
            target_runtime=runtime,
        )
    assert isinstance(export_result, ExportResult)
    profile_job = export_result.profile_job
    assert profile_job is not None
    if is_hub_testing_async():
        with open(get_profile_job_ids_file(), "a") as f:
            job_name = get_async_job_cache_name(
                path=scorecard_path,
                model_id="resnet18_quantized",
                device=device,
            )
            f.write(f"{job_name}: {profile_job.job_id}\n")
    else:
        job_result = profile_job.wait()
        assert job_result.success


NUM_EVAL_SAMPLES = 100


@pytest.mark.parametrize(
    "runtime",
    [
        TargetRuntime.TFLITE,
        TargetRuntime.QNN,
        TargetRuntime.ONNX,
    ],
)
@pytest.mark.inference
def test_inference_val_data(runtime: TargetRuntime) -> None:
    compile_job_ids = load_yaml(get_compile_job_ids_file())
    compile_job_id = get_async_job_id(
        cache=compile_job_ids,
        path=runtime,
        model_id="resnet18_quantized",
        device=cs_8_gen_3,
        fallback_to_universal_device=True,
    )
    assert compile_job_id is not None
    compile_job = hub.get_job(compile_job_id)
    assert isinstance(compile_job, hub.CompileJob)

    hub_val_dataset = get_hub_val_dataset(
        "imagenet",
        get_dataset_ids_file(),
        Model,
        apply_channel_transpose=runtime.channel_last_native_execution,
        num_samples=NUM_EVAL_SAMPLES,
    )
    ijob = hub.submit_inference_job(
        device=cs_8_gen_3.execution_device,
        inputs=hub_val_dataset,
        model=compile_job.get_target_model(),
        name="resnet18_quantized",
    )
    assert isinstance(ijob, hub.InferenceJob)
    append_line_to_file(
        get_inference_job_ids_file(),
        f"resnet18_quantized_val_{runtime.name}: {ijob.job_id}",
    )


# TODO: Add type annotations to generated tests #13595
@pytest.mark.inference
def test_val_data_torch():
    inputs, *_ = next(iter(get_torch_val_dataloader("imagenet", NUM_EVAL_SAMPLES)))
    model = Model.from_pretrained()
    output_names = model.get_output_names()
    all_outputs = [[] for _ in output_names]
    for input_tensor in inputs.split(1, dim=0):
        model_outputs = model(input_tensor)
        if isinstance(model_outputs, tuple):
            for i, out in enumerate(model_outputs):
                all_outputs[i].append(out.numpy())
        else:
            all_outputs[0].append(model_outputs.numpy())
    hub_entries = dict(zip(output_names, all_outputs))
    hub_dataset = hub.upload_dataset(hub_entries)
    append_line_to_file(
        get_dataset_ids_file(),
        f"resnet18_quantized_torch_val: {hub_dataset.dataset_id}",
    )


@pytest.fixture(scope="module")
def torch_val_outputs() -> list[np.ndarray]:
    dataset_ids = load_yaml(get_dataset_ids_file())
    dataset = hub.get_dataset(dataset_ids["resnet18_quantized_torch_val"])
    downloaded_dataset = dataset.download()
    assert isinstance(downloaded_dataset, Mapping)
    return [
        np.concatenate(tensor_list, axis=0)
        for tensor_list in downloaded_dataset.values()
    ]


@pytest.fixture(scope="module")
def torch_evaluate_mock_outputs(
    torch_val_outputs: list[np.ndarray],
) -> list[torch.Tensor | tuple[torch.Tensor, ...]]:
    num_outputs = len(torch_val_outputs)
    if num_outputs == 1:
        return [torch.Tensor(tensor).unsqueeze(0) for tensor in torch_val_outputs[0]]
    output_list = []
    for i in range(len(torch_val_outputs[0])):
        output_list.append(
            tuple(
                torch.Tensor(curr_output_list[i]).unsqueeze(0)
                for curr_output_list in torch_val_outputs
            )
        )
    return output_list


@pytest.mark.parametrize(
    "runtime",
    [
        TargetRuntime.TFLITE,
        TargetRuntime.QNN,
        TargetRuntime.ONNX,
    ],
)
@pytest.mark.export
def test_val_accuracy(
    runtime: TargetRuntime, torch_val_outputs, torch_evaluate_mock_outputs
) -> None:
    dataset_name = "imagenet"
    dataset_dir = get_and_sync_datasets_cache_dir(
        runtime.channel_last_native_execution, dataset_name, NUM_EVAL_SAMPLES
    )
    cache_path_patch = mock.patch(
        "qai_hub_models.utils.evaluate.get_hub_datasets_path",
        return_value=dataset_dir.parent,
    )
    inference_job_ids = load_yaml(get_inference_job_ids_file())
    inference_job = hub.get_job(
        inference_job_ids[f"resnet18_quantized_val_{runtime.name}"]
    )
    assert isinstance(inference_job, hub.InferenceJob)
    device_outputs = inference_job.download_output_data()
    mock_dataset_download = mock.patch(
        "qai_hub.client.Dataset.download",
        side_effect=[device_outputs],
    )
    AsyncOnDeviceModel.__call__ = mock_on_device_model_call(inference_job)  # type: ignore
    mock_torch_call = mock.patch(
        "qai_hub_models.utils.evaluate.BaseModel.__call__",
        side_effect=torch_evaluate_mock_outputs,
    )
    with cache_path_patch, mock_torch_call, mock_dataset_download:
        torch_acc, device_acc = evaluate_on_dataset(
            compiled_model=inference_job.model,
            torch_model=Model.from_pretrained(),
            hub_device=inference_job.device,
            dataset_name=dataset_name,
            split_size=NUM_EVAL_SAMPLES,
            num_samples=NUM_EVAL_SAMPLES,
            use_cache=True,
        )

    mock_torch_inference = mock.patch(
        "qai_hub_models.utils.compare._torch_inference_impl",
        side_effect=[torch_val_outputs],
    )
    mock_dataset_download = mock.patch(
        "qai_hub.client.InferenceJob.download_output_data",
        side_effect=[device_outputs],
    )
    mock_inference_job = mock.patch(
        "qai_hub.submit_inference_job",
        side_effect=[inference_job],
    )
    mock_compile_job = mock.patch(
        "qai_hub.submit_compile_job",
        return_value=inference_job.model.producer,
    )
    psnr_values = []

    def _mock_tabulate_fn(df, **kwargs) -> str:
        nonlocal psnr_values
        new_psnr_values, tabulate_results = mock_tabulate_fn(df)
        psnr_values.extend(new_psnr_values)
        return tabulate_results

    mock_tabulate = mock.patch(
        "qai_hub_models.utils.printing.tabulate",
        side_effect=_mock_tabulate_fn,
    )
    quantize_patch = mock.patch("qai_hub.submit_quantize_job", return_value=mock.Mock())
    data_patch = mock.patch(
        "qai_hub_models.models.resnet18_quantized.export.get_calibration_data",
        mock.Mock(),
    )
    with mock_torch_inference, mock_dataset_download, mock_inference_job, mock_compile_job, mock_tabulate, quantize_patch, data_patch:
        export_model(target_runtime=runtime, skip_downloading=True, skip_profiling=True)

    write_accuracy("resnet18_quantized", runtime, psnr_values, torch_acc, device_acc)


@pytest.mark.parametrize(
    "runtime",
    [
        TargetRuntime.TFLITE,
        TargetRuntime.QNN,
        TargetRuntime.ONNX,
    ],
)
@pytest.mark.export
def test_export(runtime: TargetRuntime) -> None:
    assert is_hub_testing_async(), "Must test export async."

    compile_job_ids = load_yaml(get_compile_job_ids_file())
    profile_job_ids = load_yaml(get_profile_job_ids_file())
    compile_job_id = get_async_job_id(
        cache=compile_job_ids,
        path=runtime,
        model_id="resnet18_quantized",
        device=cs_8_gen_3,
        fallback_to_universal_device=True,
    )
    assert compile_job_id is not None
    profile_job_name = get_async_job_cache_name(
        path=runtime,
        model_id="resnet18_quantized",
        device=cs_8_gen_3,
    )
    profile_job_id = profile_job_ids.get(profile_job_name, None)

    compile_job = cast(hub.CompileJob, hub.get_job(compile_job_id))
    profile_job = cast(hub.ProfileJob, hub.get_job(profile_job_id))
    compile_patch = mock.patch("qai_hub.submit_compile_job", return_value=compile_job)
    profile_patch = mock.patch("qai_hub.submit_profile_job", return_value=profile_job)
    quantize_patch = mock.patch("qai_hub.submit_quantize_job", return_value=mock.Mock())
    data_patch = mock.patch(
        "qai_hub_models.models.resnet18_quantized.export.get_calibration_data",
        mock.Mock(),
    )
    with compile_patch, profile_patch, quantize_patch, data_patch:
        export_model(
            target_runtime=runtime, skip_downloading=True, skip_inferencing=True
        )


# Trace the model & upload to hub only once for all module + input pairs.
# This speeds up tests and limits memory leaks for torch jit trace.
@pytest.fixture(scope="module", autouse=True)
def cached_torch_trace_for_export():
    with pytest.MonkeyPatch.context() as mp:
        model_cache = {}
        torch_trace = torch.jit.trace

        def _cached_torch_trace(module, inputs, check_trace=True) -> hub.Model:
            model_key = str(module) + str(
                sorted([f"{x.shape}{x.type()}" for x in inputs])
            )
            model = model_cache.get(model_key, None)
            if not model:
                trace = torch_trace(module, inputs, check_trace=check_trace)
                model = hub.upload_model(trace)
                model_cache[model_key] = model
            assert isinstance(model, hub.Model)
            return model

        mp.setattr(model_export_module.torch.jit, "trace", _cached_torch_trace)
        yield mp
