# ---------------------------------------------------------------------
# Copyright (c) 2024 Qualcomm Innovation Center, Inc. All rights reserved.
# SPDX-License-Identifier: BSD-3-Clause
# ---------------------------------------------------------------------
from qai_hub_models.scorecard.internal.model_config import ScorecardModelConfig
from qai_hub_models.utils.asset_loaders import load_yaml
from qai_hub_models.utils.path_helpers import QAIHM_PACKAGE_ROOT

MODELS_DIR_PATH = QAIHM_PACKAGE_ROOT / "scorecard" / "internal" / "models"


def test_load_validate_example_onnx():
    x = ScorecardModelConfig.from_yaml(
        MODELS_DIR_PATH / "_mobilenetv2_onnx_example.yaml"
    )
    vaildate_result = x.validate()
    assert (
        vaildate_result is None
    ), f"Mobilenet-ONNX Example Config Validation Failed: {vaildate_result}"


def test_load_validate_example_torchscript():
    x = ScorecardModelConfig.from_yaml(
        MODELS_DIR_PATH / "_mobilenetv2_torchscript_example.yaml"
    )
    vaildate_result = x.validate()
    assert (
        vaildate_result is None
    ), f"Mobilenet-Torchscript Example Config Validation Failed: {vaildate_result}"


def test_yaml_roundtrip():
    # Read from YAML and export back to YAML.
    # Compare both YAML dictionaries to make sure they're the same.
    x_dict = load_yaml(MODELS_DIR_PATH / "_mobilenetv2_torchscript_example.yaml")

    # Dict is copied because the constructor modifies it in place
    x = ScorecardModelConfig.from_dict(x_dict.copy())

    # Roundtrip back to dict
    roundtrip_dict = x.to_dict(include_defaults=False, yaml_compatible=True)

    # These were set to the default values and will
    # therefore be excluded from the roundtripped dict.
    del x_dict["channel_first_outputs"]
    del x_dict["fully_quantized"]

    assert x_dict == roundtrip_dict
