# ---------------------------------------------------------------------
# Copyright (c) 2024 Qualcomm Innovation Center, Inc. All rights reserved.
# SPDX-License-Identifier: BSD-3-Clause
# ---------------------------------------------------------------------
import pytest
import torch

from qai_hub_models.utils._internal.quantization_torch import (
    QuantizableMixin,
    module_is_quantized,
)
from qai_hub_models.utils.base_model import BaseModel


class ToyDenseNet(BaseModel, QuantizableMixin):
    def __init__(self):
        super().__init__()
        self.fc = torch.nn.Linear(10, 5)

    def forward(self, x):
        x = self.fc(x)
        return x

    @staticmethod
    def get_input_spec():
        return {"x": ((2, 10), "float32")}

    @staticmethod
    def get_output_names():
        return ["hello"]

    def from_pretrained(self):
        pass


class ToyReluNet(torch.nn.Module):
    def forward(self, x):
        return torch.nn.functional.relu(x)


@pytest.mark.parametrize(
    "module_cls,detectable",
    [
        (ToyDenseNet, True),
        (ToyReluNet, False),
    ],
)
def test_module_is_quantized(module_cls, detectable):
    model = module_cls()

    model.eval()
    assert not module_is_quantized(model)

    engine = "qnnpack"
    torch.backends.quantized.engine = engine
    model.qconfig = torch.ao.quantization.get_default_qconfig(engine)

    torch.ao.quantization.prepare(model, inplace=True)
    torch.ao.quantization.convert(model, inplace=True)
    if detectable:
        assert module_is_quantized(model)
    else:
        assert not module_is_quantized(model)


def test_dummy_ptq():
    net = ToyDenseNet()
    with pytest.warns(UserWarning, match=r".*model will not be numerically accurate.*"):
        net.post_train_quantization()
    assert module_is_quantized(net)
